@@ -1,4 +1,5 @@
 import inspect
+from jsonobject.exceptions import DeleteNotAllowed
 
 
 class JsonProperty(object):
@@ -147,7 +148,7 @@ class JsonArray(list):
 class SimpleDict(dict):
     """
     Re-implements destructive methods of dict
-    to use only setitem and getitem
+    to use only setitem and getitem and delitem
     """
     def update(self, E=None, **F):
         for dct in (E, F):
@@ -155,10 +156,14 @@ class SimpleDict(dict):
                 for key, value in dct.items():
                     self[key] = value
 
+    def clear(self):
+        for key in self.keys():
+            del self[key]
+
 
 class JsonDict(SimpleDict):
 
-    def __init__(self, _obj=None, wrapper=None, **kwargs):
+    def __init__(self, _obj=None, wrapper=None):
         super(JsonDict, self).__init__()
         self._obj = check_type(_obj, dict, 'JsonDict must wrap a dict or None')
         self._wrapper = wrapper or DefaultProperty()
@@ -177,6 +182,10 @@ class JsonDict(SimpleDict):
         self._obj[key] = unwrapped
         super(JsonDict, self).__setitem__(key, wrapped)
 
+    def __delitem__(self, key):
+        del self._obj[key]
+        super(JsonDict, self).__delitem__(key)
+
 
 class JsonObjectMeta(type):
     # There's a pretty fundamental cyclic dependency between this metaclass
@@ -286,6 +295,9 @@ class JsonObject(SimpleDict):
         return property_.unwrap(value)
 
     def __setitem__(self, key, value):
+        if key not in self._properties_by_key:
+            assert key not in self._properties_by_attr
+            super(JsonObject, self).__setattr__(key, value)
         wrapped, unwrapped = self.__unwrap(key, value)
         super(JsonObject, self).__setitem__(key, wrapped)
         self._obj[key] = unwrapped
@@ -295,6 +307,20 @@ class JsonObject(SimpleDict):
             self[name] = value
         super(JsonObject, self).__setattr__(name, value)
 
+    def __delitem__(self, key):
+        if key in self._properties_by_key:
+            raise DeleteNotAllowed(key)
+        else:
+            del self._obj[key]
+            super(JsonObject, self).__delitem__(key)
+            super(JsonObject, self).__delattr__(key)
+
+    def __delattr__(self, name):
+        if name in self._properties_by_attr:
+            raise DeleteNotAllowed(name)
+        else:
+            del self[name]
+
     def __repr__(self):
         name = self.__class__.__name__
         predefined_properties = self._properties_by_attr.keys()
